//+------------------------------------------------------------------+
//|                                                  Shaolin Quantum |
//|                                      Copyright 2015, Alex Lewis  |
//+------------------------------------------------------------------+ 
#property strict
const string ver = "1.3.2_DayOption";
input int magic_num=2015; //Magic Number
input string time_open_str="21:15"; // Trading Window Start Time (GMT)
input string time_close_str="23:45"; // Trading Window End Time (GMT)
input int max_trades=5;//Max Trades Per Trading Window (0 = unlimited)
input int max_cycles=1;//Max Cycles Per Trading Window (0 = unlimited)
input bool mon=true;//Trade on Monday
input bool tue=true;//Trade on Tuesday
input bool wed=true;//Trade on Wednesday
input bool thurs=true;//Trade on Thursday
input bool fri=false;//Trade on Friday
input int qde=500;//Quantum eintDepth3 for Entry
input int qdc=500;//Quantum eintDepth3 for Close
input int slip=10;//Order Slippage
input string awa="------------------------------------------------------";
input string axa="-------  CHOOSE LOT WISELY AND MARKET ENTRY    -------";
input string aya="------------------------------------------------------";
input double lot1=0.2; //Lot Trade 1
input double lot2=0.2; //Lot Trade 2
input double lot3=0.5; //Lot Trade 3
input double lot4=0.1; //Lot Trade 4
input double lot5=0.1; //Lot Trade 5
input double lot6=0.1; //Lot Trade 6
input double lot7=0.1; //Lot Trade 7
input double lot8=0.1; //Lot Trade 8
input double lot9=0.1; //Lot Trade 9
input double lot10=0.1; //Lot Trade 10 & >
input double sl_pct=0;//% Equity Stop Loss For All Trades (positive number Eg 2.5)
input double tp_pct=0;//% Equity Take Profit For All Trades (positive number Eg 2.5)
input double sl_dollar=0;//$ Amount Stop Loss For All Trades (positive number Eg 100.00)
input double tp_dollar=0;//$ Amount Take Profit For All Trades (positive number Eg 100.00)
input int sl_points=0;// Stop Loss Points Per Trade (Eg 50 = 5 pips)
input int tp_points=50;// Take Profit Points Per Trade (Eg 50 = 5 pips)
input int min_price_diff=10;//Minimum Points Between Trades (Eg 50 = 5 pips)

int cycles = 0;
int trades = 0;
int trade_side = -1;
datetime day_curr;
datetime day_prev; 
int bars = Bars;
double lastTradePrice = 0;

int init()
{
   return 0;
}

int deinit()
{
   return 0;
}

int start()
{
   datetime time_curr = TimeGMT();
   datetime time_open = StrToTime(time_open_str);
   datetime time_close = StrToTime(time_close_str);
   MqlDateTime dtct, dto, dtc;
   TimeToStruct(time_curr, dtct);
   TimeToStruct(time_open, dto);
   TimeToStruct(time_close, dtc);
   dto.year = dtc.year = dtct.year;
   dto.mon = dtc.mon = dtct.mon;
   dto.day = dtc.day = dtct.day;
   time_open = StructToTime(dto);
   time_close = StructToTime(dtc);
   
   if (time_open > time_close) {
      //assume next day
      time_close += 86400;
   }
   
   Comment("Shaolin Quantum v",ver,"\nGMT_time: ",time_curr,"\nTime Open (GMT): ",time_open,"\nTime Close (GMT): ",time_close);
   
   bool barNext = false;
   
   day_curr=iTime(Symbol(),PERIOD_D1,0);

   if(day_curr > day_prev)
   {
      cycles=0;
      day_prev=iTime(Symbol(),PERIOD_D1,0);
   }

   checkCloseTrades();
   
   if (Bars > bars) {
      bars = Bars;
      barNext = true;
   }

   bool cantrade=true;

   if ((cycles < max_cycles || max_cycles == 0) && (trades < max_trades || max_trades == 0) && time_curr >= time_open && time_curr < time_close && barNext)
   {
      int nextTicket = -1;
      bool trade = false;
      double price = 0;
      double tp = 0, sl = 0;
      if(mon==false && DayOfWeek()==1)
         cantrade=false;
      if(tue==false && DayOfWeek()==2)
         cantrade=false;
      if(wed==false && DayOfWeek()==3)
         cantrade=false;
      if(thurs==false && DayOfWeek()==4)
         cantrade=false;
      if(fri==false && DayOfWeek()==5)
         cantrade=false;
      //buy
      if(cantrade==true)
      {
      if (iCustom(Symbol(),0,"Quantum",qde,0,1) > 0 && (trade_side == -1 || trade_side == OP_BUY)) 
      {
         trade_side = OP_BUY;
         trade = true;
         price = Ask;
         if (tp_points)
            tp = price + (tp_points*Point);
         if (sl_points)
            sl = price - (sl_points*Point);
      }
      else
      //sell
      if (iCustom(Symbol(),0,"Quantum",qde,1,1) > 0 && (trade_side == -1 || trade_side == OP_SELL)) 
      {
         trade_side = OP_SELL;
         trade = true;
         price = Bid;
         if (tp_points)
            tp = price - (tp_points*Point);
         if (sl_points)
            sl = price + (sl_points*Point);
      }
      }
      //trade
      if (trade) 
      {
         //check minimum dist
         if (min_price_diff != 0) {
            double diff = MathAbs(price - lastTradePrice);
            if (diff < (min_price_diff*Point)) {
               Print("Minimum Price Diff Not Met: ",diff);
               return 0;
            }
         }
         
         nextTicket = OrderSend(Symbol(), trade_side, getLots(), price, slip, sl, tp, "SHAOLIN", magic_num, 0, 0);
         
         if(nextTicket <= -1) 
         {
            Print("OrderSend Error: ",GetLastError());
         } 
         else 
         {
            lastTradePrice = price;
            trades++;
         }
      }
   }
   
   return 0;
}

double getLots()
{
   double retlot = 0.01;

   if(trades>=0 && trades<1)
	   retlot=lot1;
   if(trades>=1 && trades<2)
		retlot=lot2;
	if(trades>=2 && trades<3)
		retlot=lot3;
   if(trades>=3 && trades<4)
		retlot=lot4;
	if(trades>=4 && trades<5)
		retlot=lot5;
	if(trades>=5 && trades<6)
		retlot=lot6;
	if(trades>=6 && trades<7)
		retlot=lot7;
	if(trades>=7 && trades<8)
		retlot=lot8;
	if(trades>=8 && trades<9)
		retlot=lot9;
	if(trades>=9)
		retlot=lot10;

   return retlot;
}

void checkCloseTrades()
{
   bool close = false;

   if (trades > 0)
   {
      if((sl_pct > 0 && AccountEquity() <= (AccountBalance() - (AccountBalance() * (sl_pct/100)))) ||
         (sl_dollar > 0 && AccountEquity() <= (AccountBalance() - sl_dollar)))
      {
         close = true;
         Print("Stoploss Triggered");
      }
      else if((tp_pct > 0 && AccountEquity() >= (AccountBalance() + (AccountBalance() * (tp_pct/100)))) ||
         (tp_dollar > 0 && AccountEquity() >= (AccountBalance() + tp_dollar)))
      {
         close = true;
         Print("Take Profit Triggered");
      }
      else
      {
         close = (iCustom(Symbol(),0,"Quantum",qdc,1,1) > 0 && trade_side == OP_BUY) || (iCustom(Symbol(),0,"Quantum",qdc,0,1) > 0 && trade_side == OP_SELL); 
      }
      
      if (close)
      {
         while (trades > 0)
         {          
            int tradeList[][2];  
            int size = 0;
            
            for(int h = OrdersTotal()-1; h >= 0; h--)
            {
               if(!OrderSelect(h,SELECT_BY_POS,MODE_TRADES))
                  continue;
               if(OrderSymbol() == Symbol() && OrderMagicNumber() == magic_num)
               {
                  size++;
                  ArrayResize(tradeList, size);
                  tradeList[size-1][0]=OrderOpenTime();
                  tradeList[size-1][1]=OrderTicket();
               }
            }
            
            if (size > 0)
            {
               ArraySort(tradeList);
               
               for(int i=0; i < size; i++)
               {
                  if (!OrderSelect(tradeList[i][1],SELECT_BY_TICKET))
                     continue;
                  if(OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),0,0))
                  {
                     trades--;
                  }
                  else
                  {
                     Print("OrderClose Error: ",GetLastError());
                  }
               }
            }
            else
            {
               trades = 0;
            }
         }
         cycles++;
         trade_side = -1;
         lastTradePrice = 0;
      }
   }
}