//+------------------------------------------------------------------+
//|                                                  Shaolin Quantum |
//|                                      Copyright 2015, Alex Lewis  |
//+------------------------------------------------------------------+ 
#property strict
input int magic_num=2015; //Magic Number
input string time_open_str="00:00"; // Trading Window Start Time (GMT)
input string time_close_str="08:00"; // Trading Window End Time (GMT)
input int max_trades=0;//Max Trades per Trading Slot (0 unlimited)
input int max_cycles=1;//Max Cycles per Trading Slot (0 unlimited)
input int qde=240;//Quantum eintDepth3 for Entry
input int qdc=240;//Quantum eintDepth3 for Close
input int slip=10;//Order Slippage
input double lot1=0.01; //Lot Trade 1
input double lot2=0.01; //Lot Trade 2
input double lot3=0.01; //Lot Trade 3
input double lot4=0.02; //Lot Trade 4
input double lot5=0.02; //Lot Trade 5
input double lot6=0.02; //Lot Trade 6
input double lot7=0.03; //Lot Trade 7
input double lot8=0.03; //Lot Trade 8
input double lot9=0.03; //Lot Trade 9
input double lot10=0.04; //Lot Trade 10
input double lot11=0.04; //Lot Trade 11
input double lot12=0.04; //Lot Trade 12
input double lot13=0.05; //Lot Trade 13 & >
input double sl_pct=0;//% Equity Stop Loss (positive number Eg 2.5)
input double tp_pct=0;//% Equity Take Profit (positive number Eg 2.5)
input double sl_dollar=0;//$ Equity Stop Loss (positive number Eg 100.00)
input double tp_dollar=0;//$ Equity Take Profit (positive number Eg 100.00)
input int min_price_diff=50;//Minimum Points Between Trades (50 = 5 pips)

int cycles = 0;
int trades = 0;
int trade_side = -1;
datetime day_curr;
datetime day_prev; 
int bars = Bars;
double lastTradePrice = 0;

int init()
{
   return 0;
}

int deinit()
{
   return 0;
}

int start()
{
   datetime currTime = TimeGMT();
   datetime time_open = StrToTime(time_open_str);
   datetime time_close = StrToTime(time_close_str);
   MqlDateTime dtct, dto, dtc;
   TimeToStruct(currTime, dtct);
   TimeToStruct(time_open, dto);
   TimeToStruct(time_close, dtc);
   dto.year = dtc.year = dtct.year;
   dto.mon = dtc.mon = dtct.mon;
   dto.day = dtc.day = dtct.day;
   time_open = StructToTime(dto);
   time_close = StructToTime(dtc);
   
   Comment("currTime: "+currTime+"\ntime_open: "+time_open+"\ntime_close: "+time_close);
   
   bool barNext = false;
   
   day_curr=iTime(Symbol(),PERIOD_D1,0);

   if(day_curr > day_prev)
   {
      cycles=0;
      day_prev=iTime(Symbol(),PERIOD_D1,0);
   }

   checkCloseTrades();
   
   if (Bars > bars) {
      bars = Bars;
      barNext = true;
   }

   if ((cycles < max_cycles || max_cycles == 0) && (trades < max_trades || max_trades == 0) && currTime >= time_open && currTime < time_close && barNext)
   {
      int nextTicket = -1;
      bool trade = false;
      double price = 0;
      //buy
      if (iCustom(Symbol(),0,"Quantum",qde,0,1) > 0 && (trade_side == -1 || trade_side == OP_BUY)) 
      {
         trade_side = OP_BUY;
         trade = true;
         price = Ask;
      }
      else
      //sell
      if (iCustom(Symbol(),0,"Quantum",qde,1,1) > 0 && (trade_side == -1 || trade_side == OP_SELL)) 
      {
         trade_side = OP_SELL;
         trade = true;
         price = Bid;
      }
      //trade
      if (trade) 
      {
         //check minimum dist
         if (min_price_diff != 0) {
            double diff = MathAbs(price - lastTradePrice);
            if (diff < (min_price_diff*Point)) {
               Print("Minimum Price Diff Not Met: ",diff);
               return 0;
            }
         }
         
         nextTicket = OrderSend(Symbol(), trade_side, getLots(), price, slip, 0, 0, "SHAOLIN", magic_num, 0, 0);
         
         if(nextTicket <= -1) 
         {
            Print("OrderSend Error: ",GetLastError());
         } 
         else 
         {
            lastTradePrice = price;
            trades++;
         }
      }
   }
   
   return 0;
}

double getLots()
{
   double retlot = 0.01;

   if(trades>=0 && trades<1)
      retlot=lot1;
   if(trades>=1 && trades<2)
      retlot=lot2;
   if(trades>=2 && trades<3)
      retlot=lot3;
   if(trades>=3 && trades<4)
      retlot=lot4;
   if(trades>=4 && trades<5)
      retlot=lot5;
   if(trades>=5 && trades<6)
      retlot=lot6;
   if(trades>=6 && trades<7)
      retlot=lot7;
   if(trades>=7 && trades<8)
      retlot=lot8;
   if(trades>=8 && trades<9)
      retlot=lot9;
   if(trades>=9 && trades<10)
      retlot=lot10;
   if(trades>=10 && trades<11)
      retlot=lot11;
   if(trades>=11 && trades<12)
      retlot=lot12;
   if(trades>=12)
      retlot=lot13;
   
   return retlot;
}

void checkCloseTrades()
{
   bool close = false;

   if (trades > 0)
   {
      if((sl_pct > 0 && AccountEquity() <= (AccountBalance() - (AccountBalance() * (sl_pct/100)))) ||
         (sl_dollar > 0 && AccountEquity() <= (AccountBalance() - sl_dollar)))
      {
         close = true;
         Print("Stoploss Triggered");
      }
      else if((tp_pct > 0 && AccountEquity() >= (AccountBalance() + (AccountBalance() * (tp_pct/100)))) ||
         (tp_dollar > 0 && AccountEquity() >= (AccountBalance() + tp_dollar)))
      {
         close = true;
         Print("Take Profit Triggered");
      }
      else
      {
         close = (iCustom(Symbol(),0,"Quantum",qdc,1,1) > 0 && trade_side == OP_BUY) || (iCustom(Symbol(),0,"Quantum",qdc,0,1) > 0 && trade_side == OP_SELL); 
      }
      
      if (close)
      {
         while (trades > 0)
         {          
            int tradeList[][2];  
            int size = 0;
            
            for(int h = OrdersTotal()-1; h >= 0; h--)
            {
               if(!OrderSelect(h,SELECT_BY_POS,MODE_TRADES))
                  continue;
               if(OrderSymbol() == Symbol() && OrderMagicNumber() == magic_num)
               {
                  size++;
                  ArrayResize(tradeList, size);
                  tradeList[size-1][0]=OrderOpenTime();
                  tradeList[size-1][1]=OrderTicket();
               }
            }
            
            if (size > 0)
            {
               ArraySort(tradeList);
               
               for(int i=0; i < size; i++)
               {
                  if (!OrderSelect(tradeList[i][1],SELECT_BY_TICKET))
                     continue;
                  if(OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),0,0))
                  {
                     trades--;
                  }
                  else
                  {
                     Print("OrderClose Error: ",GetLastError());
                  }
               }
            }
            else
            {
               trades = 0;
            }
         }
         cycles++;
         trade_side = -1;
         lastTradePrice = 0;
      }
   }
}